<!--
GENERATED FILE - DO NOT EDIT
This file was generated by [MarkdownSnippets](https://github.com/SimonCropp/MarkdownSnippets).
Source File: /doc/how_tos/mdsource/ScrubNonDeterministicOutput.source.md
To change this file edit the source file and then execute ./run_markdown_templates.sh.
-->

<a id="top"></a>

# How to Scrub Non-Deterministic Output

<!-- toc -->
## Contents

  * [Introduction](#introduction)
  * [Interface](#interface)
  * [Lambda example](#lambda-example)
  * [Pre-made Scrubbers](#pre-made-scrubbers)
    * [GUID](#guid)<!-- endtoc -->

## Introduction

If you are having trouble getting tests running reproducibly, you might need to use a "scrubber" to convert the non-deterministic text to something stable.

## Interface

Fundamentally, a scrubber is function that takes a string and returns a string. You can create ones by passing in a function or a lambda. We also have some pre-made ones for your convenience.

## Lambda example

<!-- snippet: scrubbing_via_lambda -->
<a id='snippet-scrubbing_via_lambda'/></a>
```cpp
Approvals::verify(
    "1 2 3 4 5 6",
    Options().withScrubber(
        [](const std::string& t) {return StringUtils::replaceAll(t, "3", "Fizz");}
    ));
```
<sup><a href='/tests/DocTest_Tests/scrubbers/GuidScrubberTests.cpp#L78-L84' title='File snippet `scrubbing_via_lambda` was extracted from'>snippet source</a> | <a href='#snippet-scrubbing_via_lambda' title='Navigate to start of snippet `scrubbing_via_lambda`'>anchor</a></sup>
<!-- endsnippet -->

This would produce:

<!-- snippet: GuidScrubberTests.Scrubbing_via_Lambda.approved.txt -->
<a id='snippet-GuidScrubberTests.Scrubbing_via_Lambda.approved.txt'/></a>
```txt
1 2 Fizz 4 5 6
```
<sup><a href='/tests/DocTest_Tests/scrubbers/approval_tests/GuidScrubberTests.Scrubbing_via_Lambda.approved.txt#L1-L1' title='File snippet `GuidScrubberTests.Scrubbing_via_Lambda.approved.txt` was extracted from'>snippet source</a> | <a href='#snippet-GuidScrubberTests.Scrubbing_via_Lambda.approved.txt' title='Navigate to start of snippet `GuidScrubberTests.Scrubbing_via_Lambda.approved.txt`'>anchor</a></sup>
<!-- endsnippet -->

(In the real world, scrubbers are generally used to remove text that is expected to differ between test runs... Here, we use a trivial example for ease of explanation.)

## Pre-made Scrubbers

### GUID

You can scrub GUIDs by using a pointer to the function `Scrubbers::scrubGuid`, for example the following code:

<!-- snippet: guid_scrubbing -->
<a id='snippet-guid_scrubbing'/></a>
```cpp
std::string jsonFromRestCall = R"(
    {
        child: {
            id: b34b4da8-090e-49d8-bd35-7e79f633a2ea
            parent1: 2fd78d4a-ad49-447d-96a8-deda585a9aa5
            parent2: 05f77de3-3790-4d45-b045-def96c9cd371
        }
        person: {
            name: mom
            id: 2fd78d4a-ad49-447d-96a8-deda585a9aa5
        }
        person: {
            name: dad
            id: 05f77de3-3790-4d45-b045-def96c9cd371
        }
    }
    )";
Approvals::verify(jsonFromRestCall, Options().withScrubber(Scrubbers::scrubGuid));
```
<sup><a href='/tests/DocTest_Tests/scrubbers/GuidScrubberTests.cpp#L33-L52' title='File snippet `guid_scrubbing` was extracted from'>snippet source</a> | <a href='#snippet-guid_scrubbing' title='Navigate to start of snippet `guid_scrubbing`'>anchor</a></sup>
<!-- endsnippet -->

will produce:

<!-- snippet: GuidScrubberTests.Input_with_multiple_GUIDs.approved.txt -->
<a id='snippet-GuidScrubberTests.Input_with_multiple_GUIDs.approved.txt'/></a>
```txt

        {
            child: {
                id: guid_1
                parent1: guid_2
                parent2: guid_3
            }
            person: {
                name: mom
                id: guid_2
            }
            person: {
                name: dad
                id: guid_3
            }
        }
        
```
<sup><a href='/tests/DocTest_Tests/scrubbers/approval_tests/GuidScrubberTests.Input_with_multiple_GUIDs.approved.txt#L1-L17' title='File snippet `GuidScrubberTests.Input_with_multiple_GUIDs.approved.txt` was extracted from'>snippet source</a> | <a href='#snippet-GuidScrubberTests.Input_with_multiple_GUIDs.approved.txt' title='Navigate to start of snippet `GuidScrubberTests.Input_with_multiple_GUIDs.approved.txt`'>anchor</a></sup>
<!-- endsnippet -->

Notice that when GUIDs are repeated within the same file, they are replaced with the same text.

---

[Back to User Guide](/doc/README.md#top)
